#include "FormatHandler.h"
#include "DataInt8.h"
#include "DataUInt16.h"
#include "DataFloat32.h"
#include <iostream>
#include <vector>

using namespace std;

void print_data_vector(const vector<Data*>& vec)
{
	vector<Data*>::const_iterator it;
	for (it = vec.begin(); it != vec.end(); ++it) {
		cout << "type=" << (*it)->get_type() << "  name=" << 
		        (*it)->get_name() << "  len=" << (*it)->get_len(); 
		cout << "  val= { ";

		if ((*it)->get_type() == "int8") {
			DataInt8 *d = static_cast<DataInt8*>(*it);
			for (unsigned long i = 0; i < d->get_len(); i++)  
			        cout << hex << 
				        (unsigned)(d->actual_data[i] & 0xFF) <<
				        " ";
		} else if ((*it)->get_type() == "uint16") {
			DataUInt16 *d = static_cast<DataUInt16*>(*it);
			for (unsigned long i = 0; i < d->get_len(); i++)  
			        cout << hex << d->actual_data[i] <<  " ";
		} else if ((*it)->get_type() == "float32") { 
			DataFloat32 *d = static_cast<DataFloat32*>(*it);
			for (unsigned long i = 0; i < d->get_len(); i++)  
			        cout << d->actual_data[i] <<  " ";
		} else {
			cout << "Unknown data type = '" << (*it)->get_type() << 
			        "' ! Skipping" << endl;
			continue;
		}
		cout << "}" << endl;
	}
}

int main(void)
{
	// Create manually the raw data: see parse_xml_data()
	// 2 uint16    = 4 bytes
	// 1 float32   = 4 bytes
	// 4 int8      = 4 bytes
        // -----------------------
        //             = 12 bytes

	uint8_t raw_data[12] = {
		// Big-Endian: 0xABCD, 0x0123 
		// 0xAB, 0xCD, 0x01, 0x23, 
		// Little-Endian: 0xABCD, 0x0123
		0xCD, 0xAB, 0x23, 0x01, 
		// Little-Endian: 1.256 = 0x3FA0C49C (float)
		0x9C, 0xC4, 0xA0, 0x3F,  
		// Endianess doesn't matter for bytes
		0xAA, 0xBB, 0xCC, 0xDD
	};

	cout << "Example: " << __FILE__ << endl << endl;

	{
	FormatHandler formatter;
	formatter.format(raw_data, 12);
	cout << endl << "FORMATTED DATA : " << endl;
	print_data_vector(formatter.get_data_vector());
	cout << endl;
	} // ~Formatter() is called here!

	cout << endl << "Example terminates " << endl;

	return 0;
}
