#include "FactoryExample3.h"
#include <iostream>

using namespace std;

DisneyFactory::creator_type_t DisneyFactory::lookup_table[MAX_CHARACTERS] = { 
	Goofy::create, 	// GOOFY_ID = 0
	Pluto::create,  // PLUTO_ID = 1
};

DisneyProduct* DisneyFactory::create(int id)
{
	if (id >= MAX_CHARACTERS || id < 0)
		return NULL;
	return lookup_table[id]();
}

DisneyUser::DisneyUser()
{
	const int config[] = {GOOFY_ID, PLUTO_ID, PLUTO_ID, 1234/*invalid*/};

	for (size_t i = 0; i < 4; i++) {
		DisneyProduct *p = DisneyFactory::create(config[i]);
		if (p)
			toons.push_back(p);
	}
}

DisneyUser::~DisneyUser() 
{
	vector<DisneyProduct*>::iterator it;
	for (it = toons.begin(); it != toons.end(); ++it)
		delete *it;
}

void DisneyUser::animate(void)
{
	vector<DisneyProduct*>::const_iterator it;
	for (it = toons.begin(); it != toons.end(); ++it)
		cout << "Hullo, my name is " << (*it)->get_name() << endl;
}

int main(void)
{
	cout << "Example: " << __FILE__ << endl << endl;

	{
	DisneyUser u;
	u.animate();
	} // ~DisneyUser() is called here!

	cout << endl << "Example terminates " << endl;

	return 0;
}
