#include <string>
#include <vector>

enum DisneyCharacterID {
	GOOFY_ID = 0,
	PLUTO_ID,
	MAX_CHARACTERS
};

class DisneyProduct {
public:
	virtual ~DisneyProduct() {};
	inline std::string get_name(void) const { return name; };
protected:
	DisneyProduct(const std::string& n) : name(n) {};
private:
	DisneyProduct(const DisneyProduct&);
	DisneyProduct &operator=(const DisneyProduct&);
	std::string name;
};

class DisneyFactory;

class Goofy : public DisneyProduct {
private:
	Goofy() : DisneyProduct("goofy") {};
	static DisneyProduct* create(void) { return new Goofy(); };
	friend class DisneyFactory;
};

class Pluto : public DisneyProduct {
private:
	Pluto() : DisneyProduct("pluto") {};
	static DisneyProduct* create(void) { return new Pluto(); };
	friend class DisneyFactory;
};

class DisneyFactory {
public:
	typedef DisneyProduct* (*creator_type_t)(void);

	static DisneyProduct* create(int id);
private:
	static creator_type_t lookup_table[MAX_CHARACTERS];
};

class DisneyUser {
public:
	DisneyUser();
	~DisneyUser(); 
	void animate(void);
private:
	std::vector<DisneyProduct*> toons;
};
